
float3 g_CameraPosition;

float  g_Time;

matrix g_World;
matrix g_ViewProjection;
matrix g_WorldViewProjection;

float Speed = 1.0f;
float4 Color = float4( 0.0f, 1.0f, 0.0f, 1.0f );

float pi = 3.14159f;

bool g_WriteToPpTarget = false;



struct VS_INPUT
{
    float3 pos		: POSITION;
};

struct PS_INPUT
{
    float4 posP			: SV_POSITION;
    float3 posW			: TEXCOORD1;
    float3 normalW		: NORMAL;
    float4 color		: COLOR0;
};













float calcWave(float pos_y)
{
    float f = (2-abs(cos(pos_y)));
    f = f*f*f*f / 3;
            
    float waveup = 0.2*cos(pos_y*4 + g_Time*Speed);
    if (waveup < 0) waveup = 0;
    float wavedown = 0.2*cos(pos_y*4 - g_Time*Speed - pi);
    if (wavedown < 0) wavedown = 0;
    
    f += waveup + wavedown;
    
    return f;
}

float3 calcPos(float3 pos, float f)
{
	float3 posModel = pos;

	posModel.x = f * posModel.x;
	posModel.z = f * posModel.z;
	posModel.y += 0.5;
	
	return posModel;
}

float3 calcNormal(float3 pos)
{	
    float2 xz = float2(pos.x, pos.z) * 0.5;
    float y = pos.y;
    float y1 = clamp(y+0.0001,-0.5,0.5);
    float y2 = clamp(y-0.0001,-0.5,0.5);
    float f_y1 = calcWave(y1) * sqrt(0.25-y1*y1);
	float f_y2 = calcWave(y2) * sqrt(0.25-y2*y2);
	float s = (f_y2-f_y1)*10000;
	//float nx = sqrt(s*s*0.5);
	
	return normalize(pos);
	//return normalize( float3(xz.x,s,xz.y) );
}




PS_INPUT VS( VS_INPUT input)
{
    PS_INPUT output;
    
    float f = calcWave(input.pos.y);
    float3 normal = calcNormal(input.pos);
    float3 posModel = calcPos(input.pos, f);
    
    output.posW = mul( float4(posModel,1), g_World ).xyz;    
    output.posP = mul( float4(output.posW,1), g_ViewProjection ); 
	output.normalW = normalize( mul(normal, (float3x3) g_World));
	output.color = float4(0,0,0,0.3) + float4(0,0,0,0.8) * f;

    return output;
}

void PS( PS_INPUT input, 
		   out float4 color		: SV_Target0,
		   out float4 normal	: SV_Target1,
		   out float4 pos		: SV_Target2,
		   out float4 pp_normal	: SV_Target3,
		   out float4 info      : SV_Target4 )
{   
	pos = float4(input.posW,1);
    color = float4(Color.rgb,0.5);
    normal = float4(normalize(input.normalW),1);
    if ( g_WriteToPpTarget ) 
    {
		pp_normal = float4(normalize(input.normalW),1);
	}
	else pp_normal = float4(0,0,0,0);
	info = float4(input.posP.z/input.posP.w,0,0,1);
}

void PS_Opacity(PS_INPUT input, 
		   out float4 color		: SV_Target0)
{
    color = float4(Color.rgb,0.3);
}






























RasterizerState DisableCulling
{
    CullMode = NONE;
};
DepthStencilState DepthEnabling
{
    DepthEnable = TRUE;
};

BlendState SrcAlphaBlendingAdd
{
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};
BlendState NoBlend
{
    BlendEnable[0] = FALSE;
};

technique10 Default
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetDepthStencilState(DepthEnabling, 0);
       // SetRasterizerState(DisableCulling);
    }
}
technique10 Default_NoOpacity
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        SetBlendState( NoBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetDepthStencilState(DepthEnabling, 0);
       // SetRasterizerState(DisableCulling);
    }
}
technique10 Opacity
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS_Opacity() ) );
        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetDepthStencilState(DepthEnabling, 0);
        SetRasterizerState(DisableCulling);
    }
}



















float g_fExtrudeAmt = 120.0 - 0;
float g_fExtrudeBias = 0;

struct LIGHTDATA
{
	float3			Position;
	bool            CalculateShadowVolumes;
};

LIGHTDATA g_Lights[400];
int g_LightCount = 0;




struct PS_INPUT_SHADOW
{
    float4 pos			: SV_Position;
};


void DetectAndProcessSilhouette( float3 L,
                                 float3 N,         // Un-normalized triangle normal
                                 PS_INPUT v1,    // Shared vertex
                                 PS_INPUT v2,    // Shared vertex
                                 PS_INPUT vAdj,  // Adjacent triangle vertex
                                 inout TriangleStream<PS_INPUT_SHADOW> ShadowTriangleStream // triangle stream
                                 )
{
    
        float3 extrude1 = normalize(v1.posW - L);
        float3 extrude2 = normalize(v2.posW - L);
     
        float3 outpos[4];       
        outpos[0] = v1.posW + g_fExtrudeBias*extrude1;
        outpos[1] = v1.posW + g_fExtrudeAmt*extrude1;
        outpos[2] = v2.posW + g_fExtrudeBias*extrude2;
        outpos[3] = v2.posW + g_fExtrudeAmt*extrude2;
            
        // Extrude silhouette to create two new triangles
        PS_INPUT_SHADOW Out;
        for(int v=0; v<4; v++)
        {
            Out.pos = mul( float4(outpos[v],1), g_ViewProjection );
            ShadowTriangleStream.Append( Out );
        }
        ShadowTriangleStream.RestartStrip();
}

[maxvertexcount(18)]
void GS_Shadow( triangleadj PS_INPUT In[6], inout TriangleStream<PS_INPUT_SHADOW> ShadowTriangleStream )
{
    // Compute un-normalized triangle normal
    float3 N = cross( In[2].posW - In[0].posW, In[4].posW - In[0].posW );
    
    
    for ( int i_light = 0; i_light < g_LightCount ; i_light++)
    {
        if ( g_Lights[i_light].CalculateShadowVolumes )
        {
            float3 lightPos = g_Lights[i_light].Position;
            // Compute direction from this triangle to the light
            float3 lightDir = lightPos - In[0].posW;
            
            //if we're facing the light
            if( dot(N, lightDir) > 0.0f )
            {
                // For each edge of the triangle, determine if it is a silhouette edge
                DetectAndProcessSilhouette( lightPos, lightDir, In[0], In[2], In[1], ShadowTriangleStream );
                DetectAndProcessSilhouette( lightPos, lightDir, In[2], In[4], In[3], ShadowTriangleStream );
                DetectAndProcessSilhouette( lightPos, lightDir, In[4], In[0], In[5], ShadowTriangleStream );
                
                //near cap5
                PS_INPUT_SHADOW Out;
                for(int v=0; v<6; v+=2)
                {
                   float3 extrude = normalize(In[v].posW - lightPos);
                    
                    float3 pos = In[v].posW + g_fExtrudeBias*extrude;
                    Out.pos = mul( float4(pos,1), g_ViewProjection );
                    ShadowTriangleStream.Append( Out );
                }
                ShadowTriangleStream.RestartStrip();
                
                //far cap (reverse the order)
                for(int v=4; v>=0; v-=2)
                {
                    float3 extrude = normalize(In[v].posW - lightPos);
                
                    float3 pos = In[v].posW + g_fExtrudeAmt*extrude;
                    Out.pos = mul( float4(pos,1), g_ViewProjection );
                    ShadowTriangleStream.Append( Out );
                }
                ShadowTriangleStream.RestartStrip();
            }
        }
    }
}


float4 PS_Shadow(PS_INPUT_SHADOW input) : SV_Target
{   
    return float4( 0.1,0.1,0.1,0.1 );
}
DepthStencilState TwoSidedStencil
{
    DepthEnable = true;
    DepthWriteMask = ZERO;
    DepthFunc = Less;
    
    StencilEnable = true;
    StencilReadMask = 0xFFFFFFFF;
    StencilWriteMask = 0xFFFFFFFF;
    
    BackFaceStencilFunc = Always;
    BackFaceStencilDepthFail = Incr;
    BackFaceStencilPass = Keep;
    BackFaceStencilFail = Keep;
    
    FrontFaceStencilFunc = Always;
    FrontFaceStencilDepthFail = Decr;
    FrontFaceStencilPass = Keep;
    FrontFaceStencilFail = Keep;
};


BlendState AdditiveBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
    SrcBlend = ONE;
    DestBlend = ONE;
    BlendOp = SUBTRACT;
    SrcBlendAlpha = ONE;
    DestBlendAlpha = ONE;
    BlendOpAlpha = SUBTRACT;
    RenderTargetWriteMask[0] = 0x0F;
};



technique10 Shadow
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( CompileShader( gs_4_0, GS_Shadow() ) );
        SetPixelShader( CompileShader( ps_4_0, PS_Shadow() ) );
        
        SetBlendState( AdditiveBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetDepthStencilState( TwoSidedStencil, 1 ); //state, stencilref
        SetRasterizerState( DisableCulling );
    }  
}