
Texture2D g_Texture;

matrix g_World;
matrix g_ViewProjection;
matrix g_WorldViewProjection;

float g_TextureRectX;
float g_TextureRectY;
float g_TextureRectW;
float g_TextureRectH;

float g_Time;
bool g_BlinkingActive;


bool g_SamplerLow = false;

SamplerState samLinear
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Wrap;
    AddressV = Wrap;
};
SamplerState samBilear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VS_INPUT
{
    float3 pos		: POSITION;
    float3 normal	: NORMAL;
    float2 tex      : TEXCOORD0;
};

struct PS_INPUT
{
    float4 posP			: SV_POSITION;
    float4 posW			: TEXCOORD1;
    float3 normalW		: NORMAL;
    float2 tex			: TEXCOORD0;
};







PS_INPUT VS( VS_INPUT input)
{
    PS_INPUT output;
        
    output.posW = mul( float4(input.pos,1), g_World );    
    output.posP = mul( output.posW, g_ViewProjection ); 
	output.normalW = normalize( float4(mul(input.normal, (float3x3) g_World),1));
    output.tex = input.tex;

    return output;
}

void PS( PS_INPUT input, 
		   out float4 color		: SV_Target0,
		   out float4 normal	: SV_Target1,
		   out float4 pos		: SV_Target2,
		   out float4 pp_normal	: SV_Target3 )
{    
	pos = input.posW;
	if ( g_SamplerLow )
		color = g_Texture.Sample(samLinear,float2(g_TextureRectX,g_TextureRectY) + float2(input.tex.x*g_TextureRectW,input.tex.y*g_TextureRectH));
	else
		color = g_Texture.Sample(samBilear,float2(g_TextureRectX,g_TextureRectY) + float2(input.tex.x*g_TextureRectW,input.tex.y*g_TextureRectH));
    if (g_BlinkingActive) color.a = color.a * (1- abs(sin((g_Time)*2)));
    normal = float4(0,1,0,1);
}































RasterizerState DisableCulling
{
    CullMode = NONE;
};
DepthStencilState DepthEnabling
{
    DepthEnable = TRUE;
};

BlendState SrcAlphaBlendingAdd
{
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};

technique10 Default
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
        
        SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetDepthStencilState(DepthEnabling, 0);
        SetRasterizerState(DisableCulling);
    }
}